from polars import Expr


def AD(high: Expr, low: Expr, close: Expr, volume: Expr) -> Expr:
    """"""
    raise


def ADOSC(high: Expr, low: Expr, close: Expr, volume: Expr, fastperiod: int = 3, slowperiod: int = 10) -> Expr:
    """"""
    raise


def OBV(close: Expr, volume: Expr) -> Expr:
    """"""
    raise
